@@ -1,7 +1,4 @@
-# THE FOLLOWING EXAMPLE WAS CONSTRUCTED FROM
-# https://stanford.edu/~shervine/blog/pytorch-how-to-generate-data-parallel
-# AND MIGRATED TO USE lpd FOR TRAINING
-#IN THIS EXAMPLE WE WILL USE THE PYTORCH DATALOADER (AS OPPOSE TO PYTHON GENERATORS)
+# IN THIS EXAMPLE WE WILL USE THE PYTORCH DATALOADER (AS OPPOSE TO PYTHON GENERATORS)
 
 import torch as T
 import torch.nn as nn
@@ -17,30 +14,33 @@ import lpd.utils.general_utils as gu
 import lpd.utils.torch_utils as tu
 
 
-class MyDataset(Dataset):
-    'Characterizes a dataset for PyTorch'
-    def __init__(self, list_IDs, labels):
-        'Initialization'
-        self.labels = labels
-        self.list_IDs = list_IDs
+num_embeddings = 10
+num_samples_per_file = 100
+def generate_file_data():
+    # [(X,y), (X,y), (X,y), (X,y)] like [ (1,0), (5,1), (6,1) ...]
+    return [(T.randint(1,num_embeddings,(1,1)).squeeze(), T.randint(0,2,(1,1)).squeeze().float()) for _ in range(num_samples_per_file)]
 
-    def _generate_fake_sample(self, ID):
-        # YOU CAN ALSO USE T.load('data/' + ID + '.pt') IF YOU HAVE FILES PER EACH ID
-        id_as_number = int(ID[-1])
-        return T.LongTensor([id_as_number]) #FOR 'id-1' RETURN [1,1,1,1], FOR 'id-2' RETURN [2,2,2,2] AND SO ON
+
+class MyDataset(Dataset):
+    def __init__(self, file_ids):
+        self.file_ids = file_ids
+        self.idx_to_file_id = {idx:file_id for idx, file_id in enumerate(self.file_ids)}
+        self.file_id_to_sample_idx = {i:0 for i in self.file_ids}
 
     def __len__(self):
-        'Denotes the total number of samples'
-        return len(self.list_IDs)
+        return len(self.file_ids)
 
     def __getitem__(self, index):
-        'Generates one sample of data'
-        # Select sample
-        ID = self.list_IDs[index]
+        file_id = self.idx_to_file_id[index]
+        
+        data = raw_data[file_id]
+        sample_idx = self.file_id_to_sample_idx[file_id]
+        self.file_id_to_sample_idx[file_id] = (self.file_id_to_sample_idx[file_id] + 1) % num_samples_per_file
 
-        # Load data and get label
-        X = self._generate_fake_sample(ID)
-        y = self.labels[ID]
+        data_at_sample_idx = data[sample_idx]
+
+        X = data_at_sample_idx[0]
+        y = data_at_sample_idx[1]
 
         return X, y
 
@@ -48,62 +48,60 @@ class MyDataset(Dataset):
 params = {  'data_loader_params': { 
                                     'batch_size': 8,
                                     'shuffle': True,
-                                    'num_workers': 6
+                                    'num_workers': 1
                                    },
-            'D_in': 4,
             'H': 128,
             'D_out': 1,
             'embedding_dim': 64,
-            'num_epochs': 50}
+            'num_epochs': 80}
 
 # Datasets
+file_ids = [f'id-{i}' for i in range(16)]
 partition = {
                 #FOR SIMPLICITY, TRAIN/VAL/TEST WILL BE THE SAME DATA
-                'train': ['id-1', 'id-2', 'id-3', 'id-4', 'id-5', 'id-6', 'id-7', 'id-8'], 
-                'val':   ['id-1', 'id-2', 'id-3', 'id-4', 'id-5', 'id-6', 'id-7', 'id-8'],  
-                'test':  ['id-1', 'id-2', 'id-3', 'id-4', 'id-5', 'id-6', 'id-7', 'id-8']   
+                'train': file_ids, 
+                'val':   file_ids,  
+                'test':  file_ids   
             }
 
-labels = {'id-1': 0., 'id-2': 1., 'id-3': 0., 'id-4': 1., 'id-5': 0., 'id-6': 1., 'id-7': 0., 'id-8': 1.}
+raw_data = {file_id:generate_file_data() for file_id in partition['train']}
 
 # Generators
-train_dataset = MyDataset(partition['train'], labels)
+train_dataset = MyDataset(partition['train'])
 train_data_loader = DataLoader(train_dataset, **params['data_loader_params'])
 
-val_dataset = MyDataset(partition['val'], labels)
+val_dataset = MyDataset(partition['val'])
 val_data_loader = DataLoader(val_dataset, **params['data_loader_params'])
 
-test_dataset = MyDataset(partition['test'], labels)
+test_dataset = MyDataset(partition['test'])
 test_data_loader = DataLoader(test_dataset, **params['data_loader_params'])
 
 
 class Model(nn.Module):
-    def __init__(self, D_in, H, D_out, num_embeddings, embedding_dim):
+    def __init__(self, H, D_out, num_embeddings, embedding_dim):
         super(Model, self).__init__()
 
         #LAYERS
 
         self.embedding_layer = nn.Embedding(num_embeddings=num_embeddings + 1, 
                                             embedding_dim=embedding_dim)
-        # nn.init.uniform_(self.embedding_layer.weight, a=-0.05, b=0.05) # I PREFER THE INIT THAT TensorFlow DO FOR Embedding
 
         self.dense = Dense(embedding_dim, H, use_bias=True, activation=nn.ReLU())
         self.dense_out = Dense(H, D_out, use_bias=True, activation=None)
 
-    def forward(self, x):               # (batch, D_in)
-        x = self.embedding_layer(x)
+    def forward(self, x):               
+        x = self.embedding_layer(x)     # (batch, embedding_dim)
         x = self.dense(x)               # (batch, H)
-        x = self.dense_out(x)           # (batch, 1, 1)
-        x = x.squeeze(2).squeeze(1)     # (batch, 1)
-        return x #NOTICE! LOGITS OUT, NOT SIGMOID, THE SIGMOID WILL BE APPLIED IN THE LOSS HANDLER FOR THIS EXAMPLE
+        x = self.dense_out(x)           # (batch, 1)
+        x = x.squeeze()                 # (batch)
+        return x #NOTICE! LOGITS OUT, NOT SIGMOID, THE SIGMOID WILL BE APPLIED IN BCEWithLogitsLoss
 
 def get_trainer(params):
 
     device = tu.get_gpu_device_if_available()
 
     # Use the nn package to define our model and loss function.
-    num_embeddings = len(train_dataset)
-    model = Model(params['D_in'], params['H'], params['D_out'], num_embeddings, params['embedding_dim']).to(device)
+    model = Model(params['H'], params['D_out'], num_embeddings, params['embedding_dim']).to(device)
 
     loss_func = nn.BCEWithLogitsLoss().to(device)
    
@@ -144,4 +142,6 @@ def run():
 
     trainer.train(params['num_epochs'])
 
-    trainer.evaluate(test_data_loader, len(test_data_loader))
\ No newline at end of file
+    evaluation = trainer.evaluate(test_data_loader, len(test_data_loader))
+
+    print(evaluation)
\ No newline at end of file
