@@ -2,4 +2,6 @@ from lpd.enums.phase import Phase
 from lpd.enums.monitor_mode import MonitorMode
 from lpd.enums.monitor_type import MonitorType
 from lpd.enums.state import State
-from lpd.enums.stats_type import StatsType
\ No newline at end of file
+from lpd.enums.stats_type import StatsType
+from lpd.enums.confusion_matrix_based_metric import ConfusionMatrixBasedMetric
+from lpd.enums.metric_method import MetricMethod
\ No newline at end of file
