@@ -1,10 +1,11 @@
 import torch as T
 from lpd.metrics.metric_base import MetricBase
+from lpd.enums.metric_method import MetricMethod
 
 
 class CategoricalAccuracy(MetricBase):
     def __init__(self):
-        pass
+        super(CategoricalAccuracy, self).__init__(MetricMethod.MEAN)
 
     def __call__(self, y_pred: T.Tensor, y_true: T.Tensor):
         indices = T.max(y_pred, 1)[1]
